import { app } from '@/app.ts';
import { ConfigModel } from './models/model.ts';
import { ShareConfigService } from './services/share.ts';
import { oss } from '@/app.ts';
import { ConfigOssService } from '@kevisual/oss/services';
import { User } from '@/models/user.ts';
import { defaultKeys } from './models/default-keys.ts';
app
  .route({
    path: 'config',
    key: 'defaultConfig',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { configKey } = ctx.query;
    if (!configKey) {
      ctx.throw(400, 'configKey is required');
    }
    const user = new User();
    user.setTokenUser(tokenUser);
    const isAdmin = await user.hasUser('admin');
    const usersConfig = ['upload.json', 'workspace.json', 'ai.json'];
    const adminConfig = ['vip.json'];
    const configs = [...usersConfig, ...(isAdmin ? adminConfig : [])];
    if (!configs.includes(configKey)) {
      ctx.throw(400, 'configKey is invalid');
    }
    const defaultConfig = defaultKeys.find((item) => item.key === configKey);

    const [config, created] = await ConfigModel.findOrCreate({
      where: {
        key: configKey,
        uid: tokenUser.id,
      },
      defaults: {
        key: configKey,
        uid: tokenUser.id,
        data: defaultConfig?.data,
      },
    });

    ctx.body = config;
  })
  .addTo(app);
